{ lib
, nixpkgsOnBuildForBuild
, nixpkgsOnBuildForHost
, version              ? throw "you must override version with the version of the linux kernel source"
, config               ? throw "you must override config with a file containing your kernel .config"
, buildTargets         ? throw "you must override buildTargets with a list of kernel Makefile targets"
}:

let
  inherit (nixpkgsOnBuildForHost) stdenv;
  version = "5.10.148";
  hash = "sha256-ElI/a+IhBnCU7FQbGWvOcEk/f0TwMbPEcR2C4pDG6VY=";
in stdenv.mkDerivation {
  pname = "linux";
  inherit version;
  src = nixpkgsOnBuildForBuild.fetchurl {
    url = "mirror://kernel/linux/kernel/v5.x/linux-${version}.tar.xz";
    inherit hash;
  };
  enableParallelBuilding = true;
  prePatch = "patchShebangs scripts";
  configurePhase = ''
    cat ${config} > .config
    echo CONFIG_EXTRA_FIRMWARE_DIR=\"${nixpkgsOnBuildForHost.linux-firmware}/lib/firmware/\" >> .config
    echo CONFIG_GCC_PLUGINS=n >> .config

    # FIXME: remove this after reenable the other space-saving
    # techniques needed to get everything into 8mbytes
    echo CC_OPTIMIZE_FOR_SIZE=y >> .config
    #echo CONFIG_CC_OPTIMIZE_FOR_PERFORMANCE=y >> .config

    make olddefconfig CROSS_COMPILE=${stdenv.cc.targetPrefix} ARCH=${stdenv.hostPlatform.linuxArch}
    runHook preConfigure
    mkdir build
    runHook postConfigure
  '';
  depsBuildBuild = [ nixpkgsOnBuildForBuild.stdenv.cc ];
  nativeBuildInputs =
    with nixpkgsOnBuildForBuild;
    [ perl bc nettools openssl rsync gmp libmpc mpfr gawk zstd python3Minimal
      kmod bison flex elfutils cpio pahole zlib elfutils ];
  hardeningDisable = [ "all" ];

  # crypto/jitterentropy.c:54:3: error: #error "The CPU Jitter random
  # number generator must not be compiled with optimizations. See
  # documentation. Use the compiler switch -O0 for compiling
  # jitterentropy.c."
  NIX_CFLAGS_COMPILE = "";

  makeFlags = [
    "CC=${stdenv.cc}/bin/${stdenv.cc.targetPrefix}cc"
    "HOSTCC=${nixpkgsOnBuildForBuild.stdenv.cc}/bin/${nixpkgsOnBuildForBuild.stdenv.cc.targetPrefix}cc"
    "ARCH=${stdenv.hostPlatform.linuxArch}"
    #"V=1"
  ] ++ lib.optional (stdenv.hostPlatform != stdenv.buildPlatform) [
    "CROSS_COMPILE=${stdenv.cc.targetPrefix}"
  ] ++ [ "modules" ] ++ buildTargets;
  karch = stdenv.hostPlatform.linuxArch;

  dontPatchELF = true;
  preInstall = ''
    installFlagsArray+=("-j$NIX_BUILD_CORES")
    installFlagsArray+=("INSTALL_MOD_PATH=$out")
    mkdir -p $out

    # note that the kernel's `Makefile` rewrites the `.config`, so we
    # keep a copy of the final version
    grep -v '^#' .config | sort > $out/config
  '';
  installTargets = [ "modules_install" ];
}
